import unittest
import random
from natural_selection.genetic_algorithms import Gene, Chromosome, Individual, Island

class TestSimpleIsland(unittest.TestCase):

    def setUp(self) -> None:
        self.fitness = lambda chromosome, island, x, y: (chromosome[0].value * x) + (chromosome[1].value *y)
        g_1 = Gene("first", 1, 25, 1, random.randint)
        g_2 = Gene("second", 1, 100, 1, random.randint)
        gen = Chromosome([g_1, g_2])
        self.ind = Individual(self.fitness, name="Adam", chromosome=gen)

        self.life = Island({'x': 0.6, 'y' : 0.2})

    def test_create(self):
        self.life.create(self.ind, population_size=5)
        self.assertEquals(len(self.life.population), 5)

    def test_import_migrants(self):
        self.life.create(self.ind, population_size=5)

        fitness = lambda chromosome, island, x, y: (chromosome[0].value * x) + (chromosome[1].value * y)
        g_1 = Gene("first", 1, 10, 1, random.randint)
        g_2 = Gene("second", 1, 100, 1, random.randint)
        gen = Chromosome([g_1, g_2])

        aliens = [Individual(fitness, name="AlsoAdam", chromosome=gen)]

        self.life.import_migrants(aliens)

        self.assertEquals(len(self.life.population), 5)

        fitness = lambda chromosome, island, x, y: (chromosome[0].value * x) + (chromosome[1].value * y)
        g_1 = Gene("first", 5, 10, 1, random.randint)
        g_2 = Gene("second", 99, 100, 1, random.randint)
        gen = Chromosome([g_1, g_2])

        aliens = [Individual(fitness, name="Eve", chromosome=gen)]

        self.life.import_migrants(aliens)

        self.assertEquals(len(self.life.population), 6)

    def test_evolve_generational(self):
        self.life.create(self.ind, population_size=5, random_seed=72)
        self.life.evolve()


# -*- coding: utf-8 -*-
"""Basic methods for selection operations.
"""
__author__ = "Justin Hocking"
__copyright__ = "Copyright 2020, Zipfian Science"
__credits__ = []
__license__ = ""
__version__ = "0.0.1"
__maintainer__ = "Justin Hocking"
__email__ = "justin.hocking@zipfian.science"
__status__ = "Development"

def selection_elites_tournament(individuals : list, n : int, desc : bool = True, island=None) -> list:
    raise NotImplemented("Not yet there, but soon!")

def selection_elites_top_n(individuals : list, n : int, desc : bool = True, island=None) -> list:
    """
    A Classic top N selection function, sorted on fitness.

    Args:
        individuals (list): A list of Individuals.
        n (int): Number to select.
        desc (bool): In descending order (default = True).
        island (Island): The Island calling the method (optional, default = None).

    Returns:
        list: Top N Individuals.
    """
    def sortFitness(val):
        return val.fitness

    individuals.sort(key=sortFitness, reverse=desc)

    return individuals[0:n]

def selection_parents_two(individuals : list, island=None):
    """
    Simple function to select two parents at a time, sequentially.

    Args:
        individuals (list): A list of Individuals, specifically the selected "elites".
        island (Island): The Island calling the method (optional, default = None).

    Yields:
        list: Containing the two individuals selected for crossover.
    """
    for parent_1, parent_2 in zip(individuals[::2], individuals[1::2]):
        yield [parent_1, parent_2]

def selection_survivors(individuals : list, n : int, desc : bool = True, island=None) -> list:
    raise NotImplemented("Not yet there, but soon!")


import unittest
import random
from natural_selection.genetic_algorithms import Gene, Chromosome

class TestGenome(unittest.TestCase):

    def test_init(self):
        g = Gene("test", 3, 10, 1, random.randint)
        gen = Chromosome([g])

        self.assertGreater(len(gen.genes), 0)

    def test_append(self):
        g = Gene("test", 3, 10, 1, random.randint)
        gen = Chromosome()
        gen.append(g)

        with self.assertRaises(AssertionError) as ae:
            gen.append(1)

    def test_indexing(self):
        g = Gene("test", 3, 10, 1, random.randint)
        gen = Chromosome([g])

        self.assertEquals(gen[0], g)

        with self.assertRaises(AssertionError) as ae:
            gen[2]

        with self.assertRaises(AssertionError) as ae:
            gen[0] = 1

        gen[0] = Gene("test_other", 3, 10, 1, random.randint)

        self.assertNotEqual(gen[0], g)

    def test_len_and_str_and_iter(self):
        g_1 = Gene("test", 3, 10, 1, random.randint)
        g_2 = Gene("test_other", 4, 10, 1, random.randint)
        gen = Chromosome([g_1, g_2])

        self.assertEquals(len(gen), 2)
        self.assertEquals(str(gen), '(test:3)-(test_other:4)')

        for g in gen:
            self.assertIsInstance(g, Gene)


import unittest
import random
from natural_selection.genetic_algorithms import Gene

class TestGene(unittest.TestCase):

    def test_init(self):
        g = Gene("test", 3, 10, 1, random.randint)
        self.assertEquals(str(g), '(test:3)')

        g_2 = g.randomize_new()

        self.assertNotEqual(g, g_2)